﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NUnit.Framework;

namespace Niche.Common
{
    using NUnit.Framework.SyntaxHelpers;

    /// <summary>
    /// Tests for the behaviour of Scoped
    /// </summary>
    [TestFixture]
    public class ScopedTests
    {
        [Test]
        public void Value_InitializedToDefault()
        {
            var cursor = new Scoped<Cursor>(Cursors.Default);
            Assert.That(cursor.Value, Is.EqualTo(Cursors.Default));
        }

        [Test]
        public void Value_UpdatedInsideWith()
        {
            var cursor = new Scoped<Cursor>(Cursors.Default);
            using(cursor.With(Cursors.Arrow))
            {
                Assert.That(cursor.Value, Is.EqualTo(Cursors.Arrow));                
            }
        }

        [Test]
        public void Value_RestoredAfterWith()
        {
            var cursor = new Scoped<Cursor>(Cursors.Default);
            using (cursor.With(Cursors.Arrow))
            {
                // Nothing
            }
            Assert.That(cursor.Value, Is.EqualTo(Cursors.Default));
        }

        [Test]
        public void ValueChanged_TriggeredAtBoundsOfWith()
        {
            int eventCount = 0;
            var cursor = new Scoped<Cursor>(Cursors.Default);
            cursor.ValueChanged +=
                    (s, a) =>
                    {
                        eventCount++;
                    };

            Assert.That(eventCount, Is.EqualTo(0));
            using (cursor.With(Cursors.Arrow))
            {
                Assert.That(eventCount, Is.EqualTo(1));
            }
            Assert.That(eventCount, Is.EqualTo(2));
        }
    }
}
